import time
import requests
from typing import Optional, Dict


grant_type = "client_credentials"
client_id = "e73882bb1255b455b356124e9b92861a"
client_secret = "$2a$10$wh9qBrCZGqGDxVNOdnZ..O4Xgxd2w4HHwvdQU3wFUz2FMQaGN.uJW"
BASE_ENDPOINT = "https://api.codeswholesale.com/{}"


def fetch_token():
    """
    Get token from codewholesale api with client_id and client_secret
    :return:
    """
    end_point = "oauth/token"
    try:
        url = BASE_ENDPOINT.format(end_point)
        payload = {
            'grant_type': 'client_credentials',
            'client_id': client_id,
            'client_secret': client_secret
        }
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        response = requests.post(url=url, data=payload, headers=headers)
        data = response.json()
        if "access_token" in data:
            return data['access_token']
        else:
            raise Exception(data.get("error", "Failed to fetch Azure auth token"))
    except Exception as e:
        raise e


class CodeWholeSaleAPI:

    DEFAULT_MAX_ATTEMPTS = 1

    def __init__(self, token=None, max_attempts: int = DEFAULT_MAX_ATTEMPTS):
        self.base_api_url = BASE_ENDPOINT
        if token:
            self.token = token
        else:
            self.token = fetch_token()
        self.max_attempts = max_attempts

    def raw_query(self, endpoint: str, params: Optional[Dict] = None, http_method: str = 'get',
                  data=None, additional_headers: Optional[Dict] = None) -> requests.Response:
        """
        CodeWholeSale API query

        :param str endpoint: URL part that follows after template url
        :param Optional[Dict] params: query parameters, optional
        :param str http_method: HTTP method, 'get' by default
        :param data: request data, see requests.request() documentation, optional
        :param Optional[Dict] additional_headers: additional request headers, optional

        :return: response object
        """
        response = False
        headers = {
            'Authorization': 'Bearer ' + self.token,
            'Content-Type': 'application/json'
        }
        if additional_headers:
            headers.update(additional_headers)
        url = self.base_api_url.format(endpoint)

        attempts = 0
        try:
            while attempts < self.max_attempts:
                response = requests.request(http_method, url, params=params, headers=headers, data=data)
                if response.status_code == 503:
                    attempts += 1
                    time.sleep(2)
                    continue
                break
        except Exception as e:
            print("             ************************  :", e.args[0])
            raise e
        return response
